@@ -1,13 +1,13 @@
 package Mock::Data;
 
 # ABSTRACT: Extensible toolkit for generating mock data
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 
 use strict;
 use warnings;
 BEGIN {
-	require MRO::Compat if "$]" < '5.009005';
+#	require MRO::Compat if "$]" < '5.009005';   # now requiring v5.10 for dist, anyway
 	require mro;
 	mro::set_mro(__PACKAGE__, 'c3');
 }
@@ -212,7 +212,7 @@ Mock::Data - Extensible toolkit for generating mock data
     },
     
     # load plugins
-	with => ['Text'],  # Mock::Data::Plugin::Text
+    plugins => ['Text'],  # Mock::Data::Plugin::Text
   );
 
   # Put all your generators into a plugin for easy access
@@ -419,7 +419,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 COPYRIGHT AND LICENSE
 
