@@ -8,7 +8,7 @@ require Mock::Data::Generator;
 our @ISA= qw( Mock::Data::Generator );
 
 # ABSTRACT: Create a generator that plugs other templates into a string
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 
 sub new {
@@ -66,7 +66,7 @@ sub _parse_template {
 		push @parts, _parse_template_reference(@_);
 	}
 	# Combine adjacent scalars in the list
-	@parts= grep ref || length, @parts;
+	@parts= grep ref $_ || length, @parts;
 	for (my $i= $#parts - 1; $i >= 0; --$i) {
 		if (!ref $parts[$i] and !ref $parts[$i+1]) {
 			$parts[$i] .= splice(@parts, $i+1, 1);
@@ -74,7 +74,7 @@ sub _parse_template {
 	}
 	if ($_[0]{compile}) {
 		return @parts == 1 && !ref $parts[0]? $parts[0]
-			: sub { join '', map +(ref? $_->(@_) : $_), @parts }
+			: sub { join '', map +(ref($_)? $_->(@_) : $_), @parts }
 	} else {
 		return \@parts;
 	}
@@ -230,7 +230,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 COPYRIGHT AND LICENSE
 
