@@ -10,7 +10,7 @@ our @EXPORT_OK= qw( uniform_set weighted_set inflate_template coerce_generator m
 );
 
 # ABSTRACT: Exportable functions to assist with declaring mock data
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 
 sub uniform_set {
@@ -53,8 +53,8 @@ sub coerce_generator {
 	: Carp::croak("Don't know how to make '$spec' into a generator");
 }
 sub _maybe_coerce_set_item {
-	!ref? inflate_template($_)
-	: ref eq 'ARRAY'? Mock::Data::Set->new(items => [map &_maybe_coerce_set_item, @$_])
+	!ref($_)? inflate_template($_)
+	: ref($_) eq 'ARRAY'? Mock::Data::Set->new(items => [map &_maybe_coerce_set_item, @$_])
 	: coerce_generator($_);
 }
 
@@ -255,7 +255,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 COPYRIGHT AND LICENSE
 
